#!/usr/bin/python
# All Rights Reserved. Copyright (c) 2018, Hitachi, Ltd.       #
# Licensed Material of Hitachi, Ltd.                           #
# Reproduction, use, modification or disclosure otherwise than #
# permitted in the License Agreement is strictly prohibited.   #
from __future__ import with_statement

BASEDIR = "/home/nasroot/foscripts"

import os
import sys
import errno
import fcntl

sys.path.append(os.path.join(BASEDIR, "bin/folib"))
import Utility
from CommonLogger import log_debug, log_info, log_warning, log_error, log_exception

sys.path.append(os.path.join(BASEDIR, "bin"))
import Exceptions
import lock_common

alert_tmp_dir_path = os.path.join(BASEDIR, "tmp")

class MsgCode(object):
    BACKUP_OK = 1
    BACKUP_ERROR = 2
    SYNC_OK = 3
    SYNC_ERROR = 4
    RESOURCE_ERROR = 5
    FAILOVER_OK = 6
    FAILOVER_ERROR = 7
    MQE_ERROR = 8

MESSAGES = {
    MsgCode.BACKUP_OK: ('KAQM80100-I', "Scheduled backup of node configuration finished normally."),
    MsgCode.BACKUP_ERROR: ('KAQM80101-E', "Scheduled backup of node configuration finished with an error."),
    MsgCode.SYNC_OK: ('KAQM80102-I', "Scheduled synchronization of file systems finished normally."),
    MsgCode.SYNC_ERROR: ('KAQM80103-E', "Scheduled synchronization of file systems finished with an error."),
    MsgCode.RESOURCE_ERROR: ('KAQM80104-E', "Resource conflicts or storage shortage was detected."),
    MsgCode.FAILOVER_OK: ('KAQM80105-I', "Failover of a file system by Failover Scripts finished normally."),
    MsgCode.FAILOVER_ERROR: ('KAQM80106-E', "Failover of a file system by Failover Scripts finished with an error."),
    MsgCode.MQE_ERROR: ('KAQM80107-W', "MQE or indexing may not be enabled on the HCP system. Please enable MQE and indexing on HCP system setting (including Replica HCP if used) if not yet, and enable Search feature on the tenant.")
}

def write_message_in_file(msg_code, msg):
    msg_id = MESSAGES[msg_code][0]

    # Create a directory to store the message file if not yet
    try:
        os.mkdir(alert_tmp_dir_path)
    except EnvironmentError, e:
        if e.errno != errno.EEXIST:
            log_error("Failed to create a system directory. path=%s" % alert_tmp_dir_path)
            raise

    # Write the message to a file
    file_path = os.path.join(alert_tmp_dir_path, msg_id)
    with open(file_path, 'w') as f:
        f.write(msg)

def send_alert(msg_code, detail=None, is_tryout=False, fsname=None):
    # Sanity check
    if msg_code not in MESSAGES.keys():
        raise Exceptions.ParameterError("Unknown message ID was specified to send an alert.")

    # Build an additinal message
    msg = ""

    if is_tryout:
        msg += " (Try-out mode)"

    if fsname is not None:
        if detail is not None:
            msg += " (File system = %s, detail = %s)" % (fsname, detail)
        else:
            msg += " (File system = %s)" % fsname
    else:
        if detail is not None:
            msg += " (%s)" % detail

    log_info("Sending an alert %s ..." % MESSAGES[msg_code][0])

    # Write the message to a file and call an official program to send an alert.
    # We need to write the message down to a file because maintexec cannot pass
    # a string containing white spaces to it's main program...
    prog = "int/send_alert"
    cmd = "sudo maintexec %s %s" % (os.path.join(BASEDIR, prog), msg_code)
    lock_file = lock_common.get_alert_lock_file()
    with open(lock_file, 'w') as locker:
        fcntl.flock(locker, fcntl.LOCK_EX)
        write_message_in_file(msg_code, msg)
        log_debug("Command-Exec-Start: %s" % cmd)
        ret, out, err = Utility.run(cmd)
        log_debug("Command-Exec-End: %s, ret=%s, out=%s" % (cmd, ret, out))
        fcntl.flock(locker, fcntl.LOCK_UN)

    if ret != 0:
        log_error("Failed to send an alert. ret=%s, out=%s, err=%s" % (ret, out, err))
